% Sidrauskis monetary model (stochastic version)

% The monetary authority chooses the level of the nominal rate of interest, 
% following Taylor's rule, in whicih the rate of inflation is not an argument:
% ic(t)=rhoi*ic(t-1)+rhoy*yc(t)+epsiloni(t)
% ic: log(1+i)-log(1+iss); yc=log(y/yss) .

% The program computes a single realization of the numerical solution and
% presents a graph of the time series for the main variables

% Copyright J. Ruiz (2006)

clear

% structural parameters

nobs=300;       % Number of observations
alpha=0.36;     % Output elasticity of capital stock
delta=0.025;    % Depreciation rate of capital
theta=0.4;      % weight of the real money in utility
sigma=2;        % Inverse of intertemporal elasticity of consumption
beta=0.99;      % Time discount factor
tauy=0.2;       % Income tax rate
tauc=0.12;      % Consumption tax rate
iss=0.02;       % nominal return
rhoi=0.95;      % autoregressive parameter for nominal return rule
Ass=3;          % steady state for technology variable
rhoA=0.90;      % Autoregressive parameter for the technology variable
rhoy=0.001;     % parameter Taylor rule


% Steady state

kss=(((1-tauy)*alpha*Ass)/((1/beta)-(1-delta)))^(1/(1-alpha));
css=Ass*(kss^alpha)-delta*kss;
rss=(1/beta)-1;
mss=theta*css*(1+iss)/iss;
piss=(1+iss)/(1+rss)-1;
xss=piss;
yss=Ass*(kss^alpha);

% initial conditions

k0=kss;
b0=1;
M0=1;
i0=iss;
A0=Ass;

% innovations

stdeA=0.0001;  % standard deviation epsilon(A)
stdei=0.0001;  % standard deviation epsilon(i)
correAi=0;   % correlation epsilon(A), epsilon(i)
coveAi=stdeA*stdei*correAi;  % covariance epsilon(A), epsilon(i)
Mcov=[stdeA^2  coveAi; coveAi  stdei^2];
PCHOL=chol(Mcov);
aleatn=normrnd(0,1,nobs+100,2);

eps=PCHOL*aleatn(101:length(aleatn),:)';

eps=[1 0;0 1]*eps;
% Computing Matrix D
MD=[0                                kss                                                                    0;
    (sigma-theta*(1-sigma))/beta     (1-alpha)*((1/beta)-(1-delta))+theta*(1-sigma)*alpha*rhoy/(beta*iss)   -theta*(1-sigma)*(1-rhoi)/(beta*iss);
    0                                0                                                                      1];

% Computing Matrix G
MG=[-css                             Ass*alpha*(kss^alpha)+(1-delta)*kss     0;
    (sigma-theta*(1-sigma))/beta     0                                       0;
    0                                alpha*rhoy                              rhoi];

% Computing Matrix H, J
MH=[Ass*(kss^alpha);
    ((1/beta)-(1-delta))-theta*(1-sigma)*rhoy/(beta*iss);
    rhoy];

MJ=[0;0;1];

% Computing Matrix Gamma1, Gamma2, Gamma3
MGamma1=inv(MD)*MG;
MGamma2=rhoA.*inv(MD)*MH;
MGamma3=inv(MD)*MJ;

% Computing matrix M^(-1),  L, Q

  [evec,eval]=eig(MGamma1);
  
  eval2=diag(eval);

  [eval3,nord]=sort(abs(eval2));

  eval4=eval2(nord);

  Mlambda=diag(eval4);

  MM=evec(:,nord);

  Mm=inv(MM);

  ML=Mm*MGamma2;
  
  MQ=Mm*MGamma3;

% Computing matrix PHI1, PHI2, PHI3, PHI4, PHI5

PHI1=[Mm(1,2)-Mm(1,1)*Mm(3,2)/Mm(3,1)  Mm(1,3)-Mm(1,1)*Mm(3,3)/Mm(3,1)  -(ML(1)+(ML(3)*Mm(1,1)/(Mm(3,1)*(rhoA-Mlambda(3,3))))*(Mlambda(1,1)-rhoA));
      Mm(2,2)-Mm(2,1)*Mm(3,2)/Mm(3,1)  Mm(2,3)-Mm(2,1)*Mm(3,3)/Mm(3,1)  -(ML(2)+(ML(3)*Mm(2,1)/(Mm(3,1)*(rhoA-Mlambda(3,3))))*(Mlambda(2,2)-rhoA));
      0                                0                                1];                   
PHI2=[Mlambda(1,1)*(Mm(1,2)-Mm(1,1)*Mm(3,2)/Mm(3,1))  Mlambda(1,1)*(Mm(1,3)-Mm(1,1)*Mm(3,3)/Mm(3,1))  0;
      Mlambda(2,2)*(Mm(2,2)-Mm(2,1)*Mm(3,2)/Mm(3,1))  Mlambda(2,2)*(Mm(2,3)-Mm(2,1)*Mm(3,3)/Mm(3,1))  0;
      0                                               0                                               1];

PHI3=[MQ(1)-MQ(3)*Mm(1,1)*Mlambda(1,1)/(Mlambda(3,3)*Mm(3,1))    0;
      MQ(2)-MQ(3)*Mm(2,1)*Mlambda(2,2)/(Mlambda(3,3)*Mm(3,1))    0;
      0                                                          1];
  
PHI4=inv(PHI1)*PHI2;

PHI5=inv(PHI1)*PHI3;

% Computing control matrix: MCONT1 MCONT2

MCONT1=[-Mm(3,2)/Mm(3,1)   -Mm(3,3)/Mm(3,1)   rhoA*ML(3)/(Mm(3,1)*(rhoA-Mlambda(3,3)))];
MCONT2=[ML(3)/(Mm(3,1)*(rhoA-Mlambda(3,3)))  -MQ(3)/(Mlambda(3,3)-Mm(3,1))];

% Computing paths for state and control variables

state=[];

ctilde=[];

stateant=[log(k0/kss); log((1+i0)/(1+iss)); log(A0/Ass)];

state=[state stateant];

for t=1:nobs
    state1=PHI4*stateant+PHI5*eps(:,t);
    
    ct=log(css)+MCONT1*stateant+MCONT2*eps(:,t);
    
    stateant=state1;
    
    state=[state stateant];
    
    ctilde=[ctilde; ct];
        
end

state=state';
ktilde=log(kss)+state(:,1);
k=exp(ktilde);
Atilde=log(Ass)+state(:,3);
A=exp(Atilde);
nrtilde=log(1+iss)+state(:,2);
nr=exp(nrtilde)-1;

c=exp(ctilde);

m=theta.*((1+nr(1:length(c)))./nr(1:length(c))).*c;  % m(t)

y=A.*(k.^(alpha));  % y(t)

Ac=Atilde-ones(length(Atilde),1)*log(Ass);
kc=ktilde-ones(length(ktilde),1)*log(kss);
rtilde=log(1+rss)+(1/(1+rss))*(1-tauy)*alpha*Ass*(kss^(alpha-1)).*(rhoA.*Ac+(alpha-1).*kc);
r=exp(rtilde)-1;   % r(t)

pietilde=nrtilde(2:nobs)-rtilde(2:nobs);   % E(pitilde(t+1))


vyss=ones(size(y,1),1)*yss;
vcss=ones(size(c,1),1)*css;
vkss=ones(size(k,1),1)*kss;
vmss=ones(size(m,1),1)*mss;
vAss=ones(size(A,1),1)*Ass;
vnrss=ones(size(nr,1),1)*iss;
vrss=ones(size(r,1),1)*rss;
vpietss=ones(size(pietilde,1),1)*log(1+piss);

figure;

subplot(4,2,1)
plot([y,vyss]);
title('output')
xlabel('periods')

subplot(4,2,2)
plot([c,vcss]);
title('consumption')
xlabel('periods')

subplot(4,2,3)
plot([k,vkss]);
title('capital stock')
xlabel('periods')

subplot(4,2,4)
plot([m,vmss]);
title('real money')
xlabel('periods')

subplot(4,2,5)
plot([pietilde,vpietss]);
title('log(inflation rate) expected')
xlabel('periods')

subplot(4,2,6)
plot([r,vrss]);
title('real return')
xlabel('periods')

subplot(4,2,7)
plot([nr,vnrss]);
title('nominal return')
xlabel('periods')

subplot(4,2,8)
plot([A,vAss]);
title('technological variable')
xlabel('periods')

hold off
