% Sidrauskis monetary model (stochastic version)

% The monetary authority chooses the level of the nominal rate of interest, 
% following Taylor's rule, in whicih the rate of inflation enters as an argument:
% ic(t)=rhoi*ic(t-1)+rhopi*pic(t)+rhoy*yc(t)+epsiloni(t)
% ic: log(1+i)-log(1+iss); yc=log(y/yss); pic=E(log((1+pi)/(1+piss))) .

% The program computes a single realization of the numerical solution and
% presents a graph of the time series for the main variables

% Copyright J. Ruiz (2006)

clear

% structural parameters

nobs=300;       % Number of observations
alpha=0.36;     % Output elasticity of capital stock
delta=0.025;    % Depreciation rate of capital
theta=0.4;      % weight of real balances in utility function
sigma=2;        % Inverse of intertemporal elasticity of consumption
beta=0.99;      % Time discount factor
tauy=0.2;       % Income tax rate
tauc=0.12;      % Consumption tax rate
iss=0.02;       % nominal interest rate
rhoi=0.95;      % autoregressive parameter for nominal return rule
Ass=3;          % steady state for technology variable
rhoA=0.9;       % Autoregressive parameter for the technology variable
rhoy=0.05;     % output coefficient in Taylor rule
rhopi=1.1;     % inflation coefficient in Taylor rule

% Steady state

kss=(((1-tauy)*alpha*Ass)/((1/beta)-(1-delta)))^(1/(1-alpha));
css=Ass*(kss^alpha)-delta*kss;
rss=(1/beta)-1;
mss=theta*css*(1+iss)/iss;
piss=(1+iss)/(1+rss)-1;
xss=piss;
yss=Ass*(kss^alpha);

% initial conditions

k0=kss;
b0=1;
M0=1;
i0=iss;
A0=Ass;

% innovations

stdeA = 0.001;  % standard deviation epsilon(A)
stdei = 0.0001;  % standard deviation epsilon(i)
correAi = 0;     % correlation epsilon(A), epsilon(i)
coveAi = stdeA*stdei*correAi;  % covariance epsilon(A), epsilon(i)
Mcov = [stdei^2  coveAi; coveAi  stdeA^2];
PCHOL = chol(Mcov);
aleatn = normrnd(0,1,nobs+100,2);

eps=PCHOL*aleatn(101:length(aleatn),:)';
eps=[1 0;0 1]*eps;
% Computing Matrix B

MB=[0 rhopi rhoy*alpha rhoi];

% Computing Matrix D, J, F, G, H, L, N, Q

MD=[(sigma-theta*(1-sigma))/beta  0       (1-alpha)*((1/beta)-(1-delta))  -theta*(1-sigma)/(beta*iss);
    (sigma-theta*(1-sigma))/beta  1/beta  0                               -(theta*(1-sigma)/(beta*iss)+((1+iss)/(1+piss)));
    0                             0       kss                             0;
    0                             0       0                               1];

MJ=[((1/beta)-(1-delta))*rhoA;
    0;
    Ass*(kss^alpha);
    0];

MF=[theta*(1-sigma)/(beta*iss);
    theta*(1-sigma)/(beta*iss);
    0;
    0];

MG=[(sigma-theta*(1-sigma))/beta   0  0                                     0;
    (sigma-theta*(1-sigma))/beta   0  0                                     0;
    -css                           0  alpha*Ass*(kss^alpha)+(1-delta)*kss   0;
    0                              0  0                                     0];

MH=[0;
    0;
    0;
    1];

ML=MD+MF*MB;
MN=MG+MH*MB;
MQ=MJ+rhoA.*MH-(rhoA*rhoy).*MF;

% Computing Matrix Gamma1, Gamma2, Gamma3

MGamma1=inv(ML)*MN;
MGamma2=inv(ML)*MQ;
MGamma3=inv(ML)*MH;

% Computing matrix M^(-1),  L, Q

  [evec,eval]=eig(MGamma1);
  
  eval2=diag(eval);

  [eval3,nord]=sort(abs(eval2));

  eval4=eval2(nord);

  Mlambda=diag(eval4);

  MM=evec(:,nord);

  Mm=inv(MM);

  MPHI=Mm*MGamma2;
  
  MPSI=Mm*MGamma3;

  
% Computing matrix S, U, V, W, THETA, XI, OMEGA, CONT1 CONT2

MS=[Mm(3,1) Mm(3,2); Mm(4,1)  Mm(4,2)];
MU=[-Mm(3,3) -Mm(3,4); -Mm(4,3) -Mm(4,4)];
MV=[MPHI(3)/(rhoA-Mlambda(3,3));
    MPHI(4)/(rhoA-Mlambda(4,4))];
MW=[-MPSI(3)/Mlambda(3,3);
    -MPSI(4)/Mlambda(4,4)];
MTHETA=inv(MS)*MU;
MXI=inv(MS)*MV;
MOMEGA=inv(MS)*MW;

MCONT1=[MTHETA  rhoA.*MXI];
MCONT2=[MOMEGA MXI];

% Computing matrix Bt Dt Ft Gt Ht Jt Lt Qt Ut Vt S1 S2

MBt=[Mm(1,1) Mm(1,2); Mm(2,1) Mm(2,2)];
MDt=[Mm(1,3) Mm(1,4); Mm(2,3) Mm(2,4)];
MFt=[0 0; Mlambda(2,2)*Mm(2,1)  Mlambda(2,2)*Mm(2,2)];
MGt=[0 0; Mlambda(2,2)*Mm(2,3)  Mlambda(2,2)*Mm(2,4)];
MHt=[MPHI(1); MPHI(2)];
MJt=[MPSI(1); MPSI(2)];
MLt=MBt*MTHETA+MDt;
MQt=MFt*MTHETA+MGt;
MUt=MHt+(MFt-rhoA.*MBt)*MXI;
MVt=MJt+MFt*MOMEGA;

MS1=[inv(MLt)*MQt   rhoA.*inv(MLt)*MUt; zeros(1,2)  rhoA];

MS2=[inv(MLt)*MVt  inv(MLt)*MUt; 0  1];



% Computing paths for state and control variables

state=[];
control=[];
stateant=[log(k0/kss); log((1+i0)/(1+iss)); log(A0/Ass)];
state=[state stateant];

for t=1:nobs
    state1=MS1*stateant+MS2*eps(:,t);
    controlt=MCONT1*stateant+MCONT2*eps(:,t);
    stateant=state1;
    state=[state stateant];
    control=[control controlt];        
end

state=state';
ktilde=log(kss)+state(:,1);
k=exp(ktilde);
Atilde=log(Ass)+state(:,3);
A=exp(Atilde);
nrtilde=log(1+iss)+state(:,2);
nr=exp(nrtilde)-1;

control=control';
ctilde=log(css)+control(:,1);
c=exp(ctilde);
pitilde=log(1+piss)+control(:,2);
pit=exp(pitilde)-1;


m=theta.*((1+nr(1:length(c)))./nr(1:length(c))).*c;  % m(t)

y=A.*(k.^(alpha));  % y(t)

Ac=Atilde-ones(length(Atilde),1)*log(Ass);
kc=ktilde-ones(length(ktilde),1)*log(kss);
rtilde=log(1+rss)+(1/(1+rss))*(1-tauy)*alpha*Ass*(kss^(alpha-1)).*(rhoA.*Ac+(alpha-1).*kc);
r=exp(rtilde)-1;   % r(t)

pietilde=nrtilde(2:nobs)-rtilde(2:nobs);   % E(pitilde(t+1))



vyss=ones(size(y,1),1)*yss;
vcss=ones(size(c,1),1)*css;
vkss=ones(size(k,1),1)*kss;
vmss=ones(size(m,1),1)*mss;
vAss=ones(size(A,1),1)*Ass;
vnrss=ones(size(nr,1),1)*iss;
vrss=ones(size(r,1),1)*rss;
vpietss=ones(size(pietilde,1),1)*log(1+piss);



figure;

subplot(4,2,1)
plot([y(1:length(c)),vyss(1:length(c))]);
title('output')
xlabel('periods')

subplot(4,2,2)
plot([c,vcss]);
title('consumption')
xlabel('periods')

subplot(4,2,3)
plot([k(1:length(c)),vkss(1:length(c))]);
title('capital stock')
xlabel('periods')

subplot(4,2,4)
plot([m(1:length(c)),vmss(1:length(c))]);
title('real balances')
xlabel('periods')

subplot(4,2,5)
plot([pit(2:nobs) pietilde,vpietss]);
title('expected and actual inflation')
xlabel('periods')

subplot(4,2,6)
plot([r(1:length(c)),vrss(1:length(c))]);
title('real interest rates')
xlabel('periods')

subplot(4,2,7)
plot([nr(1:length(c)),vnrss(1:length(c))]);
title('nominal rate of interest')
xlabel('periods')

subplot(4,2,8)
plot([A(1:length(c)),vAss(1:length(c))]);
title('technological variable')
xlabel('periods')

hold off
