% Sidrauskis monetary model (stochastic version)

% The monetary authority chooses nominal interest rates according to 
% Taylor's rule:
% ic(t)=rhoi*ic(t-1)+rhopi*pic(t)+rhoy*yc(t)+epsiloni(t)
% ic: log(1+i)-log(1+iss); yc=log(y/yss); pic=E(log((1+pi)/(1+piss))) .

% This program computes as many realizations of the numerical solution as
% desired, and provides mean and standard deviations for a number of
% statistics of interest.

% For each of the two Expectations errors in the model, 'Rationality tests' 
% at the end of the program provide its average sample mean and standard 
% deviation over the set of realizations, the average four first values of 
% its autocorrelation function, and average linear correlation coefficient 
% between the expectations error and each of: output, consumption capital stock, and
% real balances

% Copyright J. Ruiz (2006)

clear

nreal=30;        % Number of realizations
nobs=300;        % Number of observations

% structural parameters

alpha=0.36;     % Output elasticity of capital stock
delta=0.025;    % Depreciation rate of capital
theta=0.4;      % weight of real balances in utility function
sigma=2;        % Inverse of intertemporal elasticity of consumption
beta=0.99;      % Time discount factor
tauy=0.2;       % Income tax rate
tauc=0.12;      % Consumption tax rate
iss=0.02;       % nominal interest rate
rhoi=0.95;      % autoregressive parameter for nominal return rule
Ass=3;          % steady state for technology variable
rhoA=0.9;       % Autoregressive parameter for the technology variable
rhoy=0.005;     % output coefficient in Taylor rule
%   The next parameter can be set to zero to solve an economy in which Taylor's rule 
%   does not include the rate of inflation
rhopi=1.1;      % inflation coefficient in Taylor rule


param=[alpha delta theta sigma beta tauy tauc iss rhoi Ass rhoA rhoy rhopi];

% Steady state

kss=(((1-tauy)*alpha*Ass)/((1/beta)-(1-delta)))^(1/(1-alpha));
css=Ass*(kss^alpha)-delta*kss;
rss=(1/beta)-1;
mss=theta*css*(1+iss)/iss;
piss=(1+iss)/(1+rss)-1;
xss=piss;
yss=Ass*(kss^alpha);
XSS=[kss css rss piss xss mss yss];

% initial conditions

k0=kss;
b0=1;
M0=1;
i0=iss;
A0=Ass;

X0=[k0 b0 M0 i0 A0];


Mmean=[];       % matrix of means 
Mstd=[];        % matrix of standard deviations 
Mcv=[];         % matrix of coefficients of variation 
Mcorry=[];      % matrix of correlations with output 
Mcorry1=[];     % matrix of correlations with output(+1) 
Mcorry2=[];     % matrix of correlations with output(+2) 
Mcorrym1=[];    % matrix of correlations with output(-1) 
Mcorrym2=[];    % matrix of correlations with output(-2)
Merror1=[];     % matrix of expectations errors 1
Merror2=[];     % matrix of expectations errors 2

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for j=1:nreal;
    j
    
    % innovations
    
    stdeA = 0.001;   % standard deviation epsilon(A)
    stdei = 0.0001;  % standard deviation epsilon(i)
    correAi = 0;     % correlation epsilon(A), epsilon(i)
    coveAi = stdeA*stdei*correAi;  % covariance epsilon(A), epsilon(i)
    Mcov = [stdei^2  coveAi; coveAi  stdeA^2];
    PCHOL = chol(Mcov);
    aleatn = normrnd(0,1,nobs+100,2);
    
    eps=PCHOL*aleatn(101:length(aleatn),:)';
    eps=[1 0;0 1]*eps;

    [y,c,k,m,A,x,nr,r,pit,pite,ferror1,ferror2]=S_i_sr(eps,X0,XSS,nobs,param);
    
    T=length(ferror1);
    c=c(1:T);
    k=k(1:T);
    y=y(1:T);
    m=m(1:T);
    A=A(1:T);
    x=x(1:T);
    nr=nr(1:T);
    r=r(1:T);
    pit=pit(1:T);
    pite=pite(1:T);
    ferror1=ferror1(1:T);
    ferror2=ferror2(1:T);
    
    V=[c k y m A x nr r pit pite];
    
    Mmean=[Mmean; mean(V)];
    Mstd=[Mstd; std(V)];
    Mcv=[Mcv; std(V)./mean(V)];
        pc=corrcoef(c,y);
        pk=corrcoef(k,y);
        pr=corrcoef(r,y);
        pA=corrcoef(A,y);
        ppit=corrcoef(pit,y);
        ppite=corrcoef(pite,y);
        pm=corrcoef(m,y);
    Mcorry=[Mcorry; pc(1,2) pk(1,2) pr(1,2) pA(1,2) ppit(1,2) ppite(1,2) pm(1,2)];   
        pc=corrcoef(c(1:T-1),y(2:T));
        pk=corrcoef(k(1:T-1),y(2:T));
        pr=corrcoef(r(1:T-1),y(2:T));
        pA=corrcoef(A(1:T-1),y(2:T));
        ppit=corrcoef(pit(1:T-1),y(2:T));
        ppite=corrcoef(pite(1:T-1),y(2:T));
        pm=corrcoef(m(1:T-1),y(2:T));
    Mcorry1=[Mcorry1; pc(1,2) pk(1,2) pr(1,2) pA(1,2) ppit(1,2) ppite(1,2) pm(1,2)];   
        pc=corrcoef(c(1:T-2),y(3:T));
        pk=corrcoef(k(1:T-2),y(3:T));
        pr=corrcoef(r(1:T-2),y(3:T));
        pA=corrcoef(A(1:T-2),y(3:T));
        ppit=corrcoef(pit(1:T-2),y(3:T));
        ppite=corrcoef(pite(1:T-2),y(3:T));
        pm=corrcoef(m(1:T-2),y(3:T));
    Mcorry2=[Mcorry2; pc(1,2) pk(1,2) pr(1,2) pA(1,2) ppit(1,2) ppite(1,2) pm(1,2)];   
        pc=corrcoef(c(2:T),y(1:T-1));
        pk=corrcoef(k(2:T),y(1:T-1));
        pr=corrcoef(r(2:T),y(1:T-1));
        pA=corrcoef(A(2:T),y(1:T-1));
        ppit=corrcoef(pit(2:T),y(1:T-1));
        ppite=corrcoef(pite(2:T),y(1:T-1));
        pm=corrcoef(m(2:T),y(1:T-1));
    Mcorrym1=[Mcorrym1; pc(1,2) pk(1,2) pr(1,2) pA(1,2) ppit(1,2) ppite(1,2) pm(1,2)];   
        pc=corrcoef(c(3:T),y(1:T-2));
        pk=corrcoef(k(3:T),y(1:T-2));
        pr=corrcoef(r(3:T),y(1:T-2));
        pA=corrcoef(A(3:T),y(1:T-2));
        ppit=corrcoef(pit(3:T),y(1:T-2));
        ppite=corrcoef(pite(3:T),y(1:T-2));
        pm=corrcoef(m(3:T),y(1:T-2));
    Mcorrym2=[Mcorrym2; pc(1,2) pk(1,2) pr(1,2) pA(1,2) ppit(1,2) ppite(1,2) pm(1,2)];   
        merr1=mean(ferror1);
        stderr1=std(ferror1);
        merr2=mean(ferror2);
        stderr2=std(ferror2);
        
        perr1_1=corrcoef(ferror1(1:T-1),ferror1(2:T));perr1_1=perr1_1(1,2);
        perr1_2=corrcoef(ferror1(1:T-2),ferror1(3:T));perr1_2=perr1_2(1,2);
        perr1_3=corrcoef(ferror1(1:T-3),ferror1(4:T));perr1_3=perr1_3(1,2);
        perr1_4=corrcoef(ferror1(1:T-4),ferror1(5:T));perr1_4=perr1_4(1,2);
        perr1_y=corrcoef(ferror1(1:T-1),y(2:T));perr1_y=perr1_y(1,2);
        perr1_c=corrcoef(ferror1(1:T-1),c(2:T));perr1_c=perr1_c(1,2);
        perr1_k=corrcoef(ferror1(1:T-1),k(2:T));perr1_k=perr1_k(1,2);
        perr1_m=corrcoef(ferror1(1:T-1),m(2:T));perr1_m=perr1_m(1,2);
        
        perr2_1=corrcoef(ferror2(1:T-1),ferror2(2:T));perr2_1=perr2_1(1,2);
        perr2_2=corrcoef(ferror2(1:T-2),ferror2(3:T));perr2_2=perr2_2(1,2);
        perr2_3=corrcoef(ferror2(1:T-3),ferror2(4:T));perr2_3=perr2_3(1,2);
        perr2_4=corrcoef(ferror2(1:T-4),ferror2(5:T));perr2_4=perr2_4(1,2);
        perr2_y=corrcoef(ferror2(1:T-1),y(2:T));perr2_y=perr2_y(1,2);
        perr2_c=corrcoef(ferror2(1:T-1),c(2:T));perr2_c=perr2_c(1,2);
        perr2_k=corrcoef(ferror2(1:T-1),k(2:T));perr2_k=perr2_k(1,2);
        perr2_m=corrcoef(ferror2(1:T-1),m(2:T));perr2_m=perr2_m(1,2);
    Merror1=[Merror1; merr1 stderr1 perr1_1 perr1_2 perr1_3 perr1_4 perr1_y perr1_c perr1_k perr1_m];
    Merror2=[Merror2; merr2 stderr2 perr2_1 perr2_2 perr2_3 perr2_4 perr2_y perr2_c perr2_k perr2_m];
   
end

    AVGm=mean(Mmean);
    AVGs=mean(Mstd);
    AVGcv=mean(Mcv);
    AVGcy=mean(Mcorry);
    AVGcy1=mean(Mcorry1);
    AVGcy2=mean(Mcorry2);
    AVGcym1=mean(Mcorrym1);
    AVGcym2=mean(Mcorrym2);
    AVGerr1=mean(Merror1);
    AVGerr2=mean(Merror2);
    
    STDm=std(Mmean)./(nreal^0.5);
    STDs=std(Mstd)./(nreal^0.5);
    STDcv=std(Mcv)./(nreal^0.5);
    STDcy=std(Mcorry)./(nreal^0.5);
    STDcy1=std(Mcorry1)./(nreal^0.5);
    STDcy2=std(Mcorry2)./(nreal^0.5);
    STDcym1=std(Mcorrym1)./(nreal^0.5);
    STDcym2=std(Mcorrym2)./(nreal^0.5);
    STDerr1=std(Merror1)./(nreal^0.5);
    STDerr2=std(Merror2)./(nreal^0.5);
    
display '__________________________________________________________________________________________________________ '
display 'mean values of per-capita variables: means and standard deviations'
[AVGm;STDm]    
display '      c         k         y         m         A         x        nr        r        pit       pite'

display '__________________________________________________________________________________________________________ '
display 'standard deviations of per-capita variables: means and standard deviations'
[AVGs;STDs]    
display '      c         k         y         m         A         x        nr        r        pit       pite'

display '__________________________________________________________________________________________________________ '
display 'coefficients of variation of per-capita variables: means and standard deviations'
[AVGcv;STDcv]    
display '      c         k         y         m         A         x        nr        r        pit       pite'

display '__________________________________________________________________________________________________________ '
display 'correlations with output: means and standard deviations'
[AVGcy;STDcy]    
display '      c         k         r         A         pit       pite     m'

display '__________________________________________________________________________________________________________ '
display 'correlations with output(+1): means and standard deviations'
[AVGcy1;STDcy1]    
display '      c         k         r         A         pit       pite     m'

display '__________________________________________________________________________________________________________ '
display 'correlations with output(+2): means and standard deviations'
[AVGcy2;STDcy2]    
display '      c         k         r         A         pit       pite     m'

display '__________________________________________________________________________________________________________ '
display 'correlations with output(-1): means and standard deviations'
[AVGcym1;STDcym1]    
display '      c         k         r         A         pit       pite     m'

display '__________________________________________________________________________________________________________ '
display 'correlations with output(-2): means and standard deviations'
[AVGcym2;STDcym2]    
display '      c         k         r         A         pit       pite     m'

display '__________________________________________________________________________________________________________ '
display 'Rationality tests. Expectations error #1'
[AVGerr1;STDerr1]    
display '    merr1     stderr1   perr1_1   perr1_2   perr1_3   perr1_4   perr1_y   perr1_c   perr1_k   perr1_m'

display '__________________________________________________________________________________________________________ '
display 'Rationality tests. Expectations error #2'
[AVGerr2;STDerr2]    
display '    merr2     stderr2   perr2_1   perr2_2   perr2_3   perr2_4   perr2_y   perr2_c   perr2_k   perr2_m'

save means.prn AVGm STDm -ascii   
save stds.prn AVGs STDs -ascii
save cvs.prn AVGcv STDcv -ascii
save cys.prn AVGcy STDcy -ascii
save cy1s.prn AVGcy1 STDcy1 -ascii
save cy2s.prn AVGcy2 STDcy2 -ascii
save cym1s.prn AVGcym1 STDcym1 -ascii
save cym2s.prn AVGcym2 STDcym2 -ascii
save errs1.prn AVGerr1 STDerr1 -ascii
save errs1.prn AVGerr1 STDerr1 -ascii    

