% Sidrauskis monetary model (deterministic version)

% The monetary authority chooses the time path for nominal balances

% Being a deterministic model, the program would produce constant time
% series for all variables if steady-state values are as initial
% conditions.

% The program should better be used starting from an initial condition 
% outside steady-state for either the level of technology, the stock of
% capital or the rate of growth of nominal balances. This can be controlled
% for in the block of instructions labelled 'Initial conditions'

% Copyright J. Ruiz (2006)

clear

% structural parameters

nobs=300;       % Number of observations
alpha=0.36;     % Output elasticity of capital stock
delta=0.025;    % Depreciation rate of capital
theta=0.4;      % weight of real money in utility function
sigma=2;        % Inverse of intertemporal elasticity of consumption
beta=0.99;      % Time discount factor
tauy=0.15;      % Income tax rate
tauc=0.12;      % Consumption tax rate
rhox=0.95;      % autoregressive parameter for the growth rate of money supply
rhoA=0.95;      % Autoregressive parameter for the technology level

% Steady state

xss=0.01;       % growth rate of money supply
Ass=3;          % steady state for technology level
kss=(((1-tauy)*alpha*Ass)/((1/beta)-(1-delta)))^(1/(1-alpha));
css=Ass*(kss^alpha)-delta*kss;
rss=(1/beta)-1;
piss=xss;
iss=(1/beta)*(1+xss)-1;
mss=theta*(1+tauc)*css/iss;
yss=Ass*(kss^alpha);

% Initial conditions

b0=1;
M0=1;
x0=xss*(1.01);   % In this version of the program, only the rate of growth of money 
                 % supply starts outside steady-state
k0=kss;          % This line could be changed as the one for x0
A0=Ass*(1.01);          % This line could be changed as the one for x0

% Computing Matrix A
MA=[kss                                          0                          0;
    (1-alpha)*((1/beta)-(1-delta))           sigma/beta                     -theta*(1-sigma)/beta;
    (1-alpha)*((1/beta)-(1-delta))*(1+xss)   theta*(1+tauc)*css/mss         1];

% Computing Matrix B
MB=[Ass*alpha*(kss^alpha)+(1-delta)*kss     -css                       0;
    0                                        sigma/beta     -theta*(1-sigma)/beta;
    0                                        0               (1/beta)*(1+xss)];

% Computing Matrix C
MC=[Ass*(kss^alpha)                                        0;
    rhoA*((1/beta)-(1-delta))                              0;
    (1+xss)*rhoA*((1/beta)-(1-delta))         (1/beta)*(1+xss)];

% Computing Matrix Atilda, Btilda, Dtilda

ZEROS32=zeros(3,2);
Mrho = [rhoA 0; 0 rhox];
MAtilda = [MA ZEROS32; ZEROS32' eye(2)];
MBtilda = [MB MC; ZEROS32' Mrho];
MDtilda = inv(MAtilda)*MBtilda;


% Computing matrix M^(-1)

% Since M is the matrix having as columns the right eigenvectors 
% of Dtilda, its inverse has as rows the left eigenvectors of Dtilda

  [evec,eval]=eig(MDtilda');
      
    ZI=[];
    ZR=[];
    AVALR=[];AVALI=[];
    for j=1:length(evec)
        ZR=[ZR,real(evec(:,j))];
        ZI=[ZI,imag(evec(:,j))];
        AVALR=[AVALR;real(eval(j,j))];
        AVALI=[AVALI;imag(eval(j,j))];
    end

% store the unstable eigenvectors in matrix ZZu

    ZZu=[];lambdau=[];
    contr=0.0;
for j=1:length(evec)
    if abs(AVALI(j))>0.000001
       if abs((AVALR(j)^2+AVALI(j)^2)^(0.5))>=1
                         lambdau=[lambdau; (AVALR(j)^2+AVALI(j)^2)^(0.5)];
	     if contr~=abs((AVALR(j)^2+AVALI(j)^2)^(0.5))
                         ZZu=[ZZu ZR(:,j) ZI(:,j)];
	     end
	     contr=abs((AVALR(j)^2+AVALI(j)^2)^(0.5));             
       end
       elseif abs(AVALR(j))>=1
           lambdau=[lambdau; AVALR(j)];
           ZZu=[ZZu,ZR(:,j)];
    end
end

% store the stable eigenvectors in matrix ZZs

    ZZs=[];lambdas=[];
    contr=0.0;
for j=1:length(evec)
    if abs(AVALI(j))>0.000001
       if abs((AVALR(j)^2+AVALI(j)^2)^(0.5))<1
                         lambdas=[lambdas; (AVALR(j)^2+AVALI(j)^2)^(0.5)];
	     if contr~=abs((AVALR(j)^2+AVALI(j)^2)^(0.5))
                         ZZs=[ZZs ZR(:,j) ZI(:,j)];
                         
	     end
	     contr=abs((AVALR(j)^2+AVALI(j)^2)^(0.5));             
       end
       elseif abs(AVALR(j))<1
           lambdas=[lambdas; AVALR(j)];
           ZZs=[ZZs,ZR(:,j)];
    end
end
  vlambda=[lambdas; lambdau];
  Mlambda=diag(vlambda);
  ZZ=[ZZs ZZu];

% Inverse of matrix M

  Mm=ZZ';
  

% Computing matrix G, H, J, L, GAMMA1, GAMMA2

MG=Mm(4:5,2:3);

MH=[Mm(4,1)  Mm(4,4) Mm(4,5);
    Mm(5,1)  Mm(5,4) Mm(5,5)];

MJ=[MDtilda(1,1) MDtilda(1,4) MDtilda(1,5);
    MDtilda(4,1) MDtilda(4,4) MDtilda(4,5);
    MDtilda(5,1) MDtilda(5,4) MDtilda(5,5)];

ML=[MDtilda(1,2) MDtilda(1,3);
    MDtilda(4,2) MDtilda(4,3);
    MDtilda(5,2) MDtilda(5,3)];

MGAMMA1=-inv(MG)*MH;
MGAMMA2=MJ-ML*inv(MG)*MH;

% Computing time paths for state and control variables

state=[];

control=[];

stateant=[log(k0/kss); log(A0/Ass); log((1+x0)/(1+xss))];

state=[state stateant];

for t=1:nobs
    state1=MGAMMA2*stateant;
    controlt=MGAMMA1*stateant;
    stateant=state1;
    
    state=[state stateant];
    control=[control controlt];
end
state=state';
control=control';
ktilde=log(kss)+state(:,1);
k=exp(ktilde);
Atilde=log(Ass)+state(:,2);
A=exp(Atilde);
xtilde=log(1+xss)+state(:,3);
x=exp(xtilde)-1;
ctilde=log(css)+control(:,1);
c=exp(ctilde);
mtilde=log(mss)+control(:,2);
m=exp(mtilde);

y=A.*(k.^(alpha));  % y(t)

nr=theta.*(c(2:nobs)./m(2:nobs));   % i(t)

r=(1-tauy)*alpha.*A.*(k.^(alpha-1))-delta;   % r(t)

pit=((1+nr)./(1+r(2:nobs)))-1;   % pi(t+1)

% Computing time paths for M(t), P(t)

M=[];
P=[];
Mant=M0;
M=[M; Mant];
P0=Mant/m(1); Pant=P0;
P=[P; Pant];

for t=1:nobs-1
    M1=(1+x(t+1))*Mant;
    P1=M1/m(t+1);
    Mant=M1;
    M=[M;Mant];
    P=[P;P1];
end


% Computing the time path for real debt 

eta=rss+0.1;
trss=0.4*yss;
b=[];b=[b; b0];
btant=b0;
for t=1:nobs-1
    bt1=(1+r(t)-eta)*btant+trss-tauy*y(t)-tauc*c(t)-((M(t+1)-M(t))/P(t));
    btant=bt1;
    b=[b; btant];
end
bss=(1/(eta-rss))*(trss-tauy*yss-tauc*css-xss*mss);

vyss=ones(size(y,1),1)*yss;
vcss=ones(size(c,1),1)*css;
vkss=ones(size(k,1),1)*kss;
vmss=ones(size(m,1),1)*mss;
vAss=ones(size(A,1),1)*Ass;
vxss=ones(size(x,1),1)*xss;
vnrss=ones(size(nr,1),1)*iss;
vrss=ones(size(r,1),1)*rss;
vpiss=ones(size(pit,1),1)*piss;
vbss=ones(size(b,1),1)*bss;
figure;

subplot(5,2,1)
plot([y,vyss]);
title('output')
xlabel('periods')

subplot(5,2,2)
plot([c,vcss]);
title('consumption')
xlabel('periods')

subplot(5,2,3)
plot([k,vkss]);
title('capital stock')
xlabel('periods')

subplot(5,2,4)
plot([m,vmss]);
title('real balances')
xlabel('periods')

subplot(5,2,5)
plot([pit,vpiss]);
title('inflation rate')
xlabel('periods')

subplot(5,2,6)
plot([r,vrss]);
title('real interest rates')
xlabel('periods')

subplot(5,2,7)
plot([nr,vnrss]);
title('nominal interest rates')
xlabel('periods')

subplot(5,2,8)
plot([A,vAss]);
title('technological variable')
xlabel('periods')

subplot(5,2,9)
plot([x,vxss]);
title('money growth')
xlabel('periods')

subplot(5,2,10)
plot([b,vbss]);
title('real debt')
xlabel('periods')

hold off
