% Newkeynesian economy
% Monetary authority chooses the sequence of nominal interest rates 
% according to a Taylor's rule:
% ic(t)=rhoi*ic(t-1)+rhopi*pic(t)+rhoy*yc(t)+epsiloni(t)
% ic==log(1+i)-log(1+iss); yc==log(y/yss); pic==log((1+pi)/(1+piss)) .

% [@ Copyright J.Ruiz 2007]

clear

% structural parameters

nobs=300;       % Number of observations
alpha=0.36;     % Output elasticity of capital stock
delta=0.025;    % Depreciation rate of capital
theta=0.4;      % weight of the real money in utility
sigma=2;        % Inverse of intertemporal elasticity of consumption
beta=0.99;      % Time discount factor
taur=0.2;       % Capital Income tax rate
tauw=0.2;       % Labor Income tax rate
tauc=0.0;       % Consumption tax rate
phi=2;          % cost of adjusting nominal price parameter
epsi=5;         % parameter of the technology of finished good-producing firm
rhoi=0.95;      % autoregressive parameter for nominal return rule
Ass=3;          % steady state for technology variable
rhoz=0.95;      % Autoregressive parameter for the technology variable
rhoy=0.05;      % parameter Taylor rule
rhopi=1.1;      % parameter Taylor rule
rhoa=0.9;       % autoregressive parameter for the preference shock
rhoe=0.9;       % autoregressive parameter for the shock to money demand
rhov=0.9;       % autoregressive parameter for the shock to the marginal efficiency of investment

% Steady state
iss=0.02;       % nominal return that the monetary authority chooses

qss=(1/(1-taur))*((1/beta)-(1-delta));
piss=beta*(1+iss)-1;
kss=(alpha*Ass*((epsi-1)/epsi)*(1/(qss)))^(1/(1-alpha));
css=Ass*(kss^alpha)-delta*kss;
mss=theta*(1+tauc)*css*(1+iss)/iss;
yss=Ass*(kss^alpha);
wss=((1-alpha)/alpha)*qss*kss;
dss=yss-wss-qss*kss;
invss=delta*kss;

zss=1;
ass=1;
ess=1;
vss=1;

% initial conditions
b0=1;
M0=1;
k0=kss;
i0=iss;
z0=zss;
a0=ass;
e0=ess;
v0=vss;

% innovations

stdez=0.0001;  % standard desviation epsilon(z)
stdei=0.0001;  % standard desviation epsilon(i)
stdea=0.0001;  % standard desviation epsilon(a)
stdee=0.0001;  % standard desviation epsilon(e)
stdev=0.0001;  % standard desviation epsilon(v)

correza=0;   % correlation epsilon(z), epsilon(a) 
correzi=0;   % correlation epsilon(z), epsilon(i)
correze=0;   % correlation epsilon(z), epsilon(e)
correzv=0;   % correlation epsilon(z), epsilon(v)
correia=0;   % correlation epsilon(a), epsilon(i)
correie=0;   % correlation epsilon(e), epsilon(i)
correiv=0;   % correlation epsilon(v), epsilon(i)
correae=0;   % correlation epsilon(a), epsilon(e)
correav=0;   % correlation epsilon(a), epsilon(v)
correev=0;   % correlation epsilon(e), epsilon(v)
coveza=stdez*stdea*correza;  % covariance epsilon(z), epsilon(a)
covezi=stdez*stdei*correzi;  % covariance epsilon(z), epsilon(i)
coveze=stdez*stdee*correze;  % covariance epsilon(z), epsilon(e)
covezv=stdez*stdev*correzv;  % covariance epsilon(z), epsilon(v)
coveia=stdei*stdea*correia;  % covariance epsilon(i), epsilon(a)
coveie=stdei*stdee*correie;  % covariance epsilon(i), epsilon(e)
coveiv=stdei*stdev*correiv;  % covariance epsilon(i), epsilon(v)
coveae=stdea*stdee*correae;  % covariance epsilon(a), epsilon(e)
coveav=stdea*stdev*correav;  % covariance epsilon(a), epsilon(v)
coveev=stdee*stdev*correev;  % covariance epsilon(e), epsilon(v)

Mcov=[stdez^2   coveza   coveze   covezv   covezi; 
      coveza   stdea^2   coveae   coveav   coveia;
      coveze   coveae    stdee^2  coveev   coveie;
      covezv   coveav    coveev   stdev^2  coveiv;
      covezi   coveia    coveie   coveiv   stdei^2];
PCHOL=chol(Mcov);
aleatn=normrnd(0,1,nobs+100,5);

eps=PCHOL*aleatn(101:length(aleatn),:)';

    contz=1; % if contz=0 then epsilon(z)=0 for all t
    conta=0; % if conta=0 then epsilon(z)=0 for all t
    conte=0; % if conte=0 then epsilon(z)=0 for all t
    contv=0; % if contv=0 then epsilon(z)=0 for all t
    conti=0; % if conti=0 then epsilon(z)=0 for all t
    
    Mcont=[contz 0 0 0 0;0 conta 0 0 0;0 0 conte 0 0; 0 0 0 contv 0; 0 0 0 0 conti];
    eps=Mcont*eps;
    
% Computing Matrix D, G, H, J

MD=[(sigma-theta*(1-sigma))/beta  theta*(1-sigma)*rhopi/(beta*iss)            -(1-taur)  theta*(1-sigma)*rhoy*alpha/(beta*iss)  -theta*(1-sigma)*(1-rhoi)/(beta*iss);
    (sigma-theta*(1-sigma))/beta  (1/beta)+theta*(1-sigma)*rhopi/(beta*iss)   0          theta*(1-sigma)*rhoy*alpha/(beta*iss)  -(theta*(1-sigma)*(1-rhoi)/(beta*iss))-1/(1+piss);
    0                             beta*phi                                    0          0                                       0;
    0                             0                                           0          1                                       0;
    0                             0                                           0          0                                       1];

MG=[(sigma-theta*(1-sigma))/beta  0      0       0                                 0;
    (sigma-theta*(1-sigma))/beta  0      0       0                                 0;
    0                             phi    1-epsi  (1-alpha)*(1-epsi)                0;
    -css/kss                      0      0       alpha*Ass*kss^(alpha-1)+1-delta   0;
    0                             rhopi  0       rhoy*alpha                        rhoi];

MH=[-theta*(1-sigma)*rhoy*rhoz*(1-alpha)/(beta*iss)   -(1-rhoa)/beta   theta*(1-sigma)*(1-rhoe)/beta  (1-rhov)/beta;
    0                                                 0               0                               (1-rhov)/beta;
    (alpha-1)*(epsi-1)                                0               0                               0;
    (1-alpha)*Ass*kss^(alpha-1)                       0               0                               delta;
    rhoy*(1-alpha)                                    0               0                               0];

MJ=[0;0;0;0;1];


% Computing Matrix Gamma1, Gamma2, Gamma3, Gamma4, Gamma5, Gamma6

MGamma1=inv(MD)*MG;
MGamma2=inv(MD)*MH;
MGamma3=inv(MD)*MJ;

Mzeros45=zeros(4,5);
MOmega=[rhoz 0    0    0;
        0    rhoa 0    0;
        0    0    rhoe 0;
        0    0    0    rhov];
MGamma4=[MGamma1 MGamma2; Mzeros45 MOmega];
Mzeros63=zeros(6,3);
MGamma5=[eye(3);Mzeros63];
Mzeros54=zeros(5,4);
Mzeros41=zeros(4,1);
MGamma6=[Mzeros54 MGamma3; eye(4) Mzeros41];

% Computing matrix M Mtilde mtilde Psi1 Psi2 Psi3 Psi4

  [evec,eval]=eig(MGamma4);
  
  eval2=diag(eval);

  [eval3,nord]=sort(abs(eval2));

  eval4=eval2(nord);

  Mlambda=diag(eval4);

  MM=evec(:,nord);

  MMtilde=[real(MM(:,1:7)) imag(MM(:,8)) MM(:,9)];
  
  Mmtilde=inv(MMtilde);
  
  MPsi1=-inv(Mmtilde(7:9,1:3))*Mmtilde(7:9,4:9);
  
  MPsi2=-inv(Mmtilde(7:9,:)*MGamma5)*(Mmtilde(7:9,:)*MGamma6);
  
  MPsi3=MGamma4(4:9,1:3)*MPsi1+MGamma4(4:9,4:9);
  
  MPsi4=MGamma6(4:9,:);
  
% Computing paths for state and control variables

state=[];

control=[];

stateant=[log(k0/kss); log(i0/iss); log(z0/zss); log(a0/ass); log(e0/ess); log(v0/vss)];

state=[state stateant];

for t=1:nobs
    state1=MPsi3*stateant+MPsi4*eps(:,t);
    control1=MPsi1*stateant;
    stateant=state1;
    state=[state stateant];
    control=[control control1];
end

%Computing capital, nominal interest rate and shocks (z,a,e,v)
state=state';
ktilde=log(kss)+state(:,1);
kt=exp(ktilde);
itilde=log(1+iss)+state(:,2);  %itilde=log(1+i(t))
it=exp(itilde)-1;
ztilde=log(zss)+state(:,3);
zt=exp(ztilde);
atilde=log(ass)+state(:,4);
at=exp(atilde);
etilde=log(ess)+state(:,5);
et=exp(etilde);
vtilde=log(vss)+state(:,6);
vt=exp(vtilde);

%Computing consumption, inflation, and real rental rate for capital
control=control';
ctilde=log(css)+control(:,1);
ct=exp(ctilde);
pitilde=log(1+piss)+control(:,2);
pit=exp(pitilde)-1;
qtilde=log(qss)+control(:,3);
qt=exp(qtilde);

%Computing output, real money, wages, dividends and investment
yt=Ass*zt.^(1-alpha).*kt.^alpha;
mt=theta*(1+tauc)*((1+it(1:length(ct)))./it(1:length(ct))).*ct;
wt=((1-alpha)/alpha)*qt.*kt(1:length(qt));
dt=yt(1:length(qt))-wt-qt.*kt(1:length(qt))-(phi/2)*yt(1:length(qt)).*(((1+pit)/(1+piss))-1).^2;
invt=(kt(2:length(kt))-(1-delta)*kt(1:length(qt)))./vt(1:length(qt));

% Computing expected inflation: E[log(1+pit(t+1))|I(t)]
piet=log(1+piss)+(1/(beta*phi))*(phi*control(:,2)-(epsi-1)*control(:,3)-...
    (1-alpha)*(epsi-1)*state(1:length(qt),1)+(1-alpha)*(epsi-1)*state(1:length(qt),3));

T=length(ct);

% Computing expectation errors

ferror1=-(at(1:T-1)./vt(1:T-1)).*(ct(1:T-1).^(-sigma)).*(mt(1:T-1).^(theta*(1-sigma))).*(et(1:T-1).^(-theta*(1-sigma)))+...
    beta.*at(2:T).*(ct(2:T).^(-sigma)).*(mt(2:T).^(theta*(1-sigma))).*(et(2:T).^(-theta*(1-sigma))).*((1-taur).*qt(2:T)+(1-delta)./vt(2:T));

ferror2=-at(1:T-1).*(ct(1:T-1).^(-sigma)).*(mt(1:T-1).^(theta*(1-sigma))).*(et(1:T-1).^(-theta*(1-sigma)))+...
    beta.*(1+it(1:T-1)).*at(2:T).*(ct(2:T).^(-sigma)).*(mt(2:T).^(theta*(1-sigma))).*(et(2:T).^(-theta*(1-sigma)))./(1+pit(2:T));

lambdat=at(1:T).*(ct(1:T).^(-sigma)).*(mt(1:T).^(theta*(1-sigma))).*(et(1:T).^(-theta*(1-sigma)));
ferror3=-phi*(((1+pit(1:T-1))/(1+piss))-1).*((1+pit(1:T-1))/(1+piss))+...
    (1-epsi)+epsi*qt(1:T-1).*kt(1:T-1)./(alpha*yt(1:T-1))+...
    beta*phi*(lambdat(2:T)./lambdat(1:T-1)).*(((1+pit(2:T))/(1+piss))-1).*((1+pit(2:T))/(1+piss)).*(yt(2:T)./yt(1:T-1));



vkss=ones(T,1)*kss;kt=kt(1:T);
viss=ones(T,1)*iss;it=it(1:T);
vcss=ones(T,1)*css;
vpiss=ones(T,1)*piss;
vqss=ones(T,1)*qss;
vyss=ones(T,1)*yss;yt=yt(1:T);
vmss=ones(T,1)*mss;
vwss=ones(T,1)*wss;
vdss=ones(T,1)*dss;
vinvss=ones(T,1)*invss;

figure;
hold on

subplot(5,2,1)
plot([yt vyss]);
title('output');
xlabel('periods');

subplot(5,2,2)
plot([ct vcss]);
title('consumption');
xlabel('periods');

subplot(5,2,3)
plot([kt vkss]);
title('stock of capital');
xlabel('periods');

subplot(5,2,4)
plot([invt vinvss]);
title('investment');
xlabel('periods');

subplot(5,2,5)
plot([mt vmss]);
title('real balances');
xlabel('periods');

subplot(5,2,6)
plot([qt vqss]);
title('real rental for capital');
xlabel('periods');

subplot(5,2,7)
plot([wt vwss]);
title('wage rate');
xlabel('periods');

subplot(5,2,8)
plot([dt vdss]);
title('dividends');
xlabel('periods');

subplot(5,2,9)
plot([pit vpiss]);
title('actual inflation');
xlabel('periods');

subplot(5,2,10)
plot([piet vpiss]);
title('expected inflation');
xlabel('periods');

hold off