% Sidrauskis monetary model (stochastic version)

% Newkeynesian economy
% Monetary authority chooses the sequence of nominal interest rates 
% according to a Taylor's rule:

% ic(t)=rhoi*ic(t-1)+rhopi*pic(t)+rhoy*yc(t)+epsiloni(t)
% ic==log(1+i)-log(1+iss); yc==log(y/yss); pic==log((1+pi)/(1+piss)) .

clear

nreal=50;       % Number of realizations
nobs=300;       % Number of observations

% structural parameters

alpha=0.36;     % Output elasticity of capital stock
delta=0.025;    % Depreciation rate of capital
theta=0.4;      % Weight of the real money in utility
sigma=2;        % Inverse of intertemporal elasticity of consumption
beta=0.99;      % Time discount factor
taur=0.2;       % Capital Income tax rate
tauw=0.2;       % Labor Income tax rate
tauc=0.0;       % Consumption tax rate
phi=2;          % Cost of adjusting nominal price parameter
epsi=5;         % Parameter of the technology of finished good-producing firm
rhoi=0.95;      % Autoregressive parameter for nominal return rule
Ass=3;          % Steady state for technology variable
rhoz=0.95;      % Autoregressive parameter for the technology variable
rhoy=0.05;      % Parameter Taylor rule
rhopi=1.1;      % Parameter Taylor rule
rhoa=0.9;       % Autoregressive parameter for the preference shock
rhoe=0.9;       % Autoregressive parameter for the shock to money demand
rhov=0.9;       % Autoregressive parameter for the shock to the marginal efficiency of investment


param=[alpha delta theta sigma beta taur tauw tauc phi epsi rhoi Ass rhoz rhoy rhopi rhoa rhoe rhov];

% Steady state
iss=0.02;       % Nominal return that the monetary authority chooses

qss=(1/(1-taur))*((1/beta)-(1-delta));
piss=beta*(1+iss)-1;
kss=(alpha*Ass*((epsi-1)/epsi)*(1/(qss)))^(1/(1-alpha));
css=Ass*(kss^alpha)-delta*kss;
mss=theta*(1+tauc)*css*(1+iss)/iss;
yss=Ass*(kss^alpha);
wss=((1-alpha)/alpha)*qss*kss;
dss=yss-wss-qss*kss;
invss=delta*kss;

zss=1;
ass=1;
ess=1;
vss=1;

XSS=[iss qss piss kss css mss yss wss dss invss zss ass ess vss];
% initial conditions
b0=1;
M0=1;
k0=kss;
i0=iss;
z0=zss;
a0=ass;
e0=ess;
v0=vss;

X0=[b0 M0 k0 i0 z0 a0 e0 v0];

Mmean=[];       % matrix of means 
Mstd=[];        % matrix of standard desviations 
Mcv=[];         % matrix of coefficients of variation 
Mcorry=[];      % matrix of correlations with output 
Mcorry1=[];     % matrix of correlations with output(+1) 
Mcorry2=[];     % matrix of correlations with output(+2) 
Mcorrym1=[];    % matrix of correlations with output(-1) 
Mcorrym2=[];    % matrix of correlations with output(-2)
Merror1=[];     % matrix of expectations errors 1
Merror2=[];     % matrix of expectations errors 2
Merror3=[];     % matrix of expectations errors 3

    datac = [];
    datak = [];
    datay = [];
    datam = [];
    datai = [];
    datainv = [];
    dataq = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for j=1:nreal
    j
    
    % innovations
    
    stdez=0.001;  % standard desviation epsilon(z)
    stdei=0.001;  % standard desviation epsilon(i)
    stdea=0.001;  % standard desviation epsilon(a)
    stdee=0.001;  % standard desviation epsilon(e)
    stdev=0.001;  % standard desviation epsilon(v)

    correza=0;   % correlation epsilon(z), epsilon(a) 
    correzi=0;   % correlation epsilon(z), epsilon(i)
    correze=0;   % correlation epsilon(z), epsilon(e)
    correzv=0;   % correlation epsilon(z), epsilon(v)
    correia=0;   % correlation epsilon(a), epsilon(i)
    correie=0;   % correlation epsilon(e), epsilon(i)
    correiv=0;   % correlation epsilon(v), epsilon(i)
    correae=0;   % correlation epsilon(a), epsilon(e)
    correav=0;   % correlation epsilon(a), epsilon(v)
    correev=0;   % correlation epsilon(e), epsilon(v)
    coveza=stdez*stdea*correza;  % covariance epsilon(z), epsilon(a)
    covezi=stdez*stdei*correzi;  % covariance epsilon(z), epsilon(i)
    coveze=stdez*stdee*correze;  % covariance epsilon(z), epsilon(e)
    covezv=stdez*stdev*correzv;  % covariance epsilon(z), epsilon(v)
    coveia=stdei*stdea*correia;  % covariance epsilon(i), epsilon(a)
    coveie=stdei*stdee*correie;  % covariance epsilon(i), epsilon(e)
    coveiv=stdei*stdev*correiv;  % covariance epsilon(i), epsilon(v)
    coveae=stdea*stdee*correae;  % covariance epsilon(a), epsilon(e)
    coveav=stdea*stdev*correav;  % covariance epsilon(a), epsilon(v)
    coveev=stdee*stdev*correev;  % covariance epsilon(e), epsilon(v)

    Mcov=[stdez^2   coveza   coveze   covezv   covezi; 
          coveza   stdea^2   coveae   coveav   coveia;
          coveze   coveae    stdee^2  coveev   coveie;
          covezv   coveav    coveev   stdev^2  coveiv;
          covezi   coveia    coveie   coveiv   stdei^2];
    PCHOL=chol(Mcov);
    aleatn=normrnd(0,1,nobs+100,5);

    eps=PCHOL*aleatn(101:length(aleatn),:)';
    
    contz=1; % if contz=0 then epsilon(z)=0 for all t
    conta=0; % if conta=0 then epsilon(z)=0 for all t
    conte=0; % if conte=0 then epsilon(z)=0 for all t
    contv=0; % if contv=0 then epsilon(z)=0 for all t
    conti=1; % if conti=0 then epsilon(z)=0 for all t
    
    Mcont=[contz 0 0 0 0;0 conta 0 0 0;0 0 conte 0 0; 0 0 0 contv 0; 0 0 0 0 conti];
    eps=Mcont*eps;
    
    
    [yt,ct,kt,mt,it,pit,piet,invt,qt,wt,dt,zt,at,et,vt,ferror1,ferror2,ferror3]=nkeyn(eps,X0,XSS,nobs,param);
    
    T=length(ferror1);
    
    yt=yt(1:T);
    ct=ct(1:T);
    kt=kt(1:T);
    mt=mt(1:T);
    it=it(1:T);
    pit=pit(1:T);
    piet=piet(1:T);
    invt=invt(1:T);
    qt=qt(1:T);
    wt=wt(1:T);
    dt=dt(1:T);
    zt=zt(1:T);
       
    datac = [datac ct];
    datak = [datak kt];
    datay = [datay yt];
    datam = [datam mt];
    datai = [datai it];
    datainv = [datainv invt];
    dataq = [dataq qt];

    V=[ct kt yt mt it pit piet invt qt zt];
    
    Mmean=[Mmean; mean(V)];
    Mstd=[Mstd; std(V)];
    Mcv=[Mcv; std(V)./mean(V)];
        pc=corrcoef(ct,yt);
        pk=corrcoef(kt,yt);
        pq=corrcoef(qt,yt);
        pz=corrcoef(zt,yt);
        ppit=corrcoef(pit,yt);
        ppite=corrcoef(piet,yt);
        pm=corrcoef(mt,yt);
    Mcorry=[Mcorry; pc(1,2) pk(1,2) pq(1,2) pz(1,2) ppit(1,2) ppite(1,2) pm(1,2)];   
        pc=corrcoef(ct(1:T-1),yt(2:T));
        pk=corrcoef(kt(1:T-1),yt(2:T));
        pq=corrcoef(qt(1:T-1),yt(2:T));
        pz=corrcoef(zt(1:T-1),yt(2:T));
        ppit=corrcoef(pit(1:T-1),yt(2:T));
        ppite=corrcoef(piet(1:T-1),yt(2:T));
        pm=corrcoef(mt(1:T-1),yt(2:T));
    Mcorry1=[Mcorry1; pc(1,2) pk(1,2) pq(1,2) pz(1,2) ppit(1,2) ppite(1,2) pm(1,2)];   
        pc=corrcoef(ct(1:T-2),yt(3:T));
        pk=corrcoef(kt(1:T-2),yt(3:T));
        pq=corrcoef(qt(1:T-2),yt(3:T));
        pz=corrcoef(zt(1:T-2),yt(3:T));
        ppit=corrcoef(pit(1:T-2),yt(3:T));
        ppite=corrcoef(piet(1:T-2),yt(3:T));
        pm=corrcoef(mt(1:T-2),yt(3:T));
    Mcorry2=[Mcorry2; pc(1,2) pk(1,2) pq(1,2) pz(1,2) ppit(1,2) ppite(1,2) pm(1,2)];   
        pc=corrcoef(ct(2:T),yt(1:T-1));
        pk=corrcoef(kt(2:T),yt(1:T-1));
        pq=corrcoef(qt(2:T),yt(1:T-1));
        pz=corrcoef(zt(2:T),yt(1:T-1));
        ppit=corrcoef(pit(2:T),yt(1:T-1));
        ppite=corrcoef(piet(2:T),yt(1:T-1));
        pm=corrcoef(mt(2:T),yt(1:T-1));
    Mcorrym1=[Mcorrym1; pc(1,2) pk(1,2) pq(1,2) pz(1,2) ppit(1,2) ppite(1,2) pm(1,2)];   
        pc=corrcoef(ct(3:T),yt(1:T-2));
        pk=corrcoef(kt(3:T),yt(1:T-2));
        pq=corrcoef(qt(3:T),yt(1:T-2));
        pz=corrcoef(zt(3:T),yt(1:T-2));
        ppit=corrcoef(pit(3:T),yt(1:T-2));
        ppite=corrcoef(piet(3:T),yt(1:T-2));
        pm=corrcoef(mt(3:T),yt(1:T-2));
    Mcorrym2=[Mcorrym2; pc(1,2) pk(1,2) pq(1,2) pz(1,2) ppit(1,2) ppite(1,2) pm(1,2)];   
        merr1=mean(ferror1);
        stderr1=std(ferror1);
        merr2=mean(ferror2);
        stderr2=std(ferror2);
        merr3=mean(ferror3);
        stderr3=std(ferror3);
        
        perr1_1=corrcoef(ferror1(1:T-1),ferror1(2:T));perr1_1=perr1_1(1,2);
        perr1_2=corrcoef(ferror1(1:T-2),ferror1(3:T));perr1_2=perr1_2(1,2);
        perr1_3=corrcoef(ferror1(1:T-3),ferror1(4:T));perr1_3=perr1_3(1,2);
        perr1_4=corrcoef(ferror1(1:T-4),ferror1(5:T));perr1_4=perr1_4(1,2);
        perr1_y=corrcoef(ferror1(1:T-1),yt(2:T));perr1_y=perr1_y(1,2);
        perr1_c=corrcoef(ferror1(1:T-1),ct(2:T));perr1_c=perr1_c(1,2);
        perr1_k=corrcoef(ferror1(1:T-1),kt(2:T));perr1_k=perr1_k(1,2);
        perr1_m=corrcoef(ferror1(1:T-1),mt(2:T));perr1_m=perr1_m(1,2);
        
        perr2_1=corrcoef(ferror2(1:T-1),ferror2(2:T));perr2_1=perr2_1(1,2);
        perr2_2=corrcoef(ferror2(1:T-2),ferror2(3:T));perr2_2=perr2_2(1,2);
        perr2_3=corrcoef(ferror2(1:T-3),ferror2(4:T));perr2_3=perr2_3(1,2);
        perr2_4=corrcoef(ferror2(1:T-4),ferror2(5:T));perr2_4=perr2_4(1,2);
        perr2_y=corrcoef(ferror2(1:T-1),yt(2:T));perr2_y=perr2_y(1,2);
        perr2_c=corrcoef(ferror2(1:T-1),ct(2:T));perr2_c=perr2_c(1,2);
        perr2_k=corrcoef(ferror2(1:T-1),kt(2:T));perr2_k=perr2_k(1,2);
        perr2_m=corrcoef(ferror2(1:T-1),mt(2:T));perr2_m=perr2_m(1,2);
        
        perr3_1=corrcoef(ferror3(1:T-1),ferror3(2:T));perr3_1=perr3_1(1,2);
        perr3_2=corrcoef(ferror3(1:T-2),ferror3(3:T));perr3_2=perr3_2(1,2);
        perr3_3=corrcoef(ferror3(1:T-3),ferror3(4:T));perr3_3=perr3_3(1,2);
        perr3_4=corrcoef(ferror3(1:T-4),ferror3(5:T));perr3_4=perr3_4(1,2);
        perr3_y=corrcoef(ferror3(1:T-1),yt(2:T));perr3_y=perr3_y(1,2);
        perr3_c=corrcoef(ferror3(1:T-1),ct(2:T));perr3_c=perr3_c(1,2);
        perr3_k=corrcoef(ferror3(1:T-1),kt(2:T));perr3_k=perr3_k(1,2);
        perr3_m=corrcoef(ferror3(1:T-1),mt(2:T));perr3_m=perr3_m(1,2);
        
    Merror1=[Merror1; merr1 stderr1 perr1_1 perr1_2 perr1_3 perr1_4 perr1_y perr1_c perr1_k perr1_m];
    Merror2=[Merror2; merr2 stderr2 perr2_1 perr2_2 perr2_3 perr2_4 perr2_y perr2_c perr2_k perr2_m];
    Merror3=[Merror3; merr3 stderr3 perr3_1 perr3_2 perr3_3 perr3_4 perr3_y perr3_c perr3_k perr3_m];
end

    AVGm=mean(Mmean);
    AVGs=mean(Mstd);
    AVGcv=mean(Mcv);
    AVGcy=mean(Mcorry);
    AVGcy1=mean(Mcorry1);
    AVGcy2=mean(Mcorry2);
    AVGcym1=mean(Mcorrym1);
    AVGcym2=mean(Mcorrym2);
    AVGerr1=mean(Merror1);
    AVGerr2=mean(Merror2);
    AVGerr3=mean(Merror3);
    
    STDm=std(Mmean)./(nreal^0.5);
    STDs=std(Mstd)./(nreal^0.5);
    STDcv=std(Mcv)./(nreal^0.5);
    STDcy=std(Mcorry)./(nreal^0.5);
    STDcy1=std(Mcorry1)./(nreal^0.5);
    STDcy2=std(Mcorry2)./(nreal^0.5);
    STDcym1=std(Mcorrym1)./(nreal^0.5);
    STDcym2=std(Mcorrym2)./(nreal^0.5);
    STDerr1=std(Merror1)./(nreal^0.5);
    STDerr2=std(Merror2)./(nreal^0.5);
    STDerr3=std(Merror3)./(nreal^0.5);
    
display '__________________________________________________________________________________________________________ '
display 'Variable means: Average and standard desviation across simulations'
[AVGm;STDm]    
display '      c         k         y         m         i         pi       pie      inv       q         z'

display '__________________________________________________________________________________________________________ '
display 'Variable standard desviations: Average and standard desviation across simulations'
[AVGs;STDs]    
display '      c         k         y         m         i         pi       pie      inv       q         z'

display '__________________________________________________________________________________________________________ '
display 'Coefficients of variation: Average and standard desviation across simulations'
[AVGcv;STDcv]    
display '      c         k         y         m         i         pi       pie      inv       q         z'

display '__________________________________________________________________________________________________________ '
display 'Correlations with output: Average and standard desviation across simulations'
[AVGcy;STDcy]    
display '      c         k         q         z         pit       pite     m'

display '__________________________________________________________________________________________________________ '
display 'Correlations with output(+1): Average and standard desviation across simulationsAverage and standard desviation across simulations'
[AVGcy1;STDcy1]    
display '      c         k         q         z         pit       pite     m'

display '__________________________________________________________________________________________________________ '
display 'Correlations with output(+2): Average and standard desviation across simulations'
[AVGcy2;STDcy2]    
display '      c         k         q         z         pit       pite     m'

display '__________________________________________________________________________________________________________ '
display 'Correlations with output(-1): Average and standard desviation across simulations'
[AVGcym1;STDcym1]    
display '      c         k         q         z         pit       pite     m'

display '__________________________________________________________________________________________________________ '
display 'Correlations with output(-2): Average and standard desviation across simulations'
[AVGcym2;STDcym2]    
display '      c         k         q         z         pit       pite     m'

display '__________________________________________________________________________________________________________ '
display 'Rationality tests'
display '(merr1,stderr1): Mean value of expectation error: Average and standard deviation across simulation'
display '(perr1_1,perr1_2,perr1_3,perr1_4): Mean value of autocorrelation of expectation error'
display '(perr1_y,perr1_c,perr1_k,perr1_m): Mean value of correlation of expectation error with y, c, k, m'
[AVGerr1;STDerr1]    
display '    merr1     stderr1   perr1_1   perr1_2   perr1_3   perr1_4   perr1_y   perr1_c   perr1_k   perr1_m'

display '__________________________________________________________________________________________________________ '
display 'Rationality tests'
display '(merr2,stderr2): Mean value of expectation error: Average and standard deviation across simulation'
display '(perr2_1,perr2_2,perr2_3,perr2_4): Mean value of autocorrelation of expectation error'
display '(perr2_y,perr2_c,perr2_k,perr2_m): Mean value of correlation of expectation error with y, c, k, m'
[AVGerr2;STDerr2]    
display '    merr2     stderr2   perr2_1   perr2_2   perr2_3   perr2_4   perr2_y   perr2_c   perr2_k   perr2_m'

display '__________________________________________________________________________________________________________ '
display 'Rationality tests'
display '(merr3,stderr3): Mean value of expectation error: Average and standard deviation across simulation'
display '(perr3_1,perr3_2,perr3_3,perr3_4): Mean value of autocorrelation of expectation error'
display '(perr3_y,perr3_c,perr3_k,perr3_m): Mean value of correlation of expectation error with y, c, k, m'
[AVGerr3;STDerr3]    
display '    merr3     stderr3   perr3_1   perr3_2   perr3_3   perr3_4   perr3_y   perr3_c   perr3_k   perr3_m'

save means.prn AVGm STDm -ascii   
save stds.prn AVGs STDs -ascii
save cvs.prn AVGcv STDcv -ascii
save cys.prn AVGcy STDcy -ascii
save cy1s.prn AVGcy1 STDcy1 -ascii
save cy2s.prn AVGcy2 STDcy2 -ascii
save cym1s.prn AVGcym1 STDcym1 -ascii
save cym2s.prn AVGcym2 STDcym2 -ascii
save errs1.prn AVGerr1 STDerr1 -ascii
save errs2.prn AVGerr2 STDerr2 -ascii    
save errs3.prn AVGerr3 STDerr3 -ascii    

    save datac.prn datac -ascii   
    save datak.prn datak -ascii   
    save datay.prn datay -ascii
    save datam.prn datam -ascii   
    save datai.prn datai -ascii   
    save datainv.prn datainv -ascii   
    save dataq.prn dataq -ascii   
   